{+doc
+NOMBRE: MatReal
+CREACION: 1990
+AUTORES: rch
+REGISTRO:
+TIPO: Unidad Pascal.
+PROPOSITO: definicion del objeto matriz de reales.
+PROYECTO: rchlib

+REVISION:
+AUTOR:
+DESCRIPCION:
	Todas las tareas implementadas en esta biblioteca, tienen como
filosofia que los datos estan bien definidos, y para mejorar la
velocidad no verifican la coherencia. Tampoco se encargan de
inicializar matrices. Supongamos que vamos a multiplicar las
matrices A y B , y queremos el resultado en C, la sentecia es
C.MultM(A,B); pero se debe cuidar que los rangos sean los adecuados
y se debe inicializar C antes del llamado a MultMatR.
	Las unicas tareas que inicializan  datos del objeto son las de
lectura. Ej: a.readM; inicialliza a.

-doc}
{-----------------------------------------------------  RCH/90  --
Ing. Ruben Chaer                                 M.E.E.P.
 Dpto. Maquinas Electricas y Electronica de Potencia
 Av. J. Herrera y Reissig No 565
------------------------------------------------------------------}
unit MatReal;

interface
uses
	{$I xobjects},
  xMatDefs,
	MatObj,ComPol;

type

	p_real = ^NReal;


	PVectR = ^TVectR;
	TVectR = object(TVect)
			constructor Init(ne:word);
			constructor Ventana( ne: word; var x );
			constructor Load( var S: TStream );
			function e(k:word):NReal;
			procedure pon_e(k:word; x:NReal);
			procedure acum_e(k:word; x:NReal);
			function SumaDeElementos: NReal;
			function PEV(var y :TVectR):NReal;
			function PEVRFLX(var y:TVectR): NReal;
			procedure PorReal(r:NReal);
			procedure sum(var y:TVectR);
			procedure sumRPV(r:NReal;var x:TVectR);
			function ne2:NReal; {norma euclideana al cuadrado }
			function normEuclid:NReal;
			function normMaxAbs:NReal;
			function normSumAbs:NReal;
			procedure Copy(var x:TVectR);
			procedure Ceros; virtual;
			procedure MinMax(var kMin, kMax:word; var Min, Max:NReal);
			procedure Print;virtual;

			{ Divide las componetes por la norma obligando al vector a
			tener norma ecuclidea = 1 }
			procedure HacerUnitario;

			(*
			function EstimFrec(
				nivel,					{nivel de compoaracion}
				histeresis: NReal;	{histeresis del cruce }
				AbajoArriba: boolean	{sentido del cruce}
				): NReal;				{cantidad de cruces}
			*)
	end;

const
	RVectR: TStreamRec = (
	 ObjType: 4005;
	 VmtLink: Ofs(TypeOf(TVectR)^);
	 Load:    @TVectR.Load;
	 Store:   @TVectR.Store
	);


type
	PMatR = ^TMatR;
	TMatR = object(TMat)
		constructor Init(filas,columnas:word);
		constructor Load( var S: TStream );
		function e(k,j:word):NReal;
		procedure pon_e(k,j: integer; x: NReal);
		procedure acum_e(k,j:integer; x: NReal);
		procedure Mult(a,b:TMatR);
		procedure Transformar( var y, x: TVectR );
		procedure WriteM;
		constructor ReadM; (* a debe estar sin inicializar *)
		function Traza:NReal;
		function Deter:NReal;
		function Escaler(var i: TMatR):NReal;
		procedure PolinomioCaracteristico(var P:TPoliR);
		procedure CopyColVect(var Y: TVectR; J: Integer);
		procedure Inv;
		procedure Ceros; virtual;
		procedure CerosFila( kfil: integer);
		procedure MinMax(
			var kMin, jMin:word;
			var kMax, jMax:word;
			var Min,  Max:NReal);

		function NormMaxAbs: NReal;


		{ OJO esta en desarrollo
		retorna la dimensin del subespacio NULO.
		La matriz deve ser cuadrada (hasta que se revise el algoritmo
		el resultado es la matriz Base, en la cual el conjunto de filas forman
		una base del subespacio nulo de la matriz SELF. Si se quiere una base
		Ortonormal invocar el mtodo Ortonormal en la matriz resultado (base).
		Como ejemplo: CBSENON.PAS.
		LAS FILAS DE Base SON LOS VECTORES DE LA BASE }
		function CalcBSEN( var Base: TMatR ): integer;

		{ Hace que el conjunto de filas sea una base OROTORMAL,
		el resultado de la funcin es la dimensin del espacio generado
		por las filas.
		Como ejemplo: CBSENON.PAS
		ORTOGONALIZA LAS FILAS DE LA MATRIZ COMO SI FUERA UNA BASE }
		function OrtoNormal: integer;


		{ Calcula una base del subespacio invariante asociado a un autovalor
		real dado 
		LAS FILAS DE Base SON LOS VECTORES DE LA BASE }
		function CalcBSE_R( var Base: TMatR; av: NReal): integer;

		{ Calcula una base del subespacio invariate asociado a un par de
		autovalores complejos conjugados
		LAS FILAS DE Base SON LOS VECTORES DE LA BASE }
		function CalcBSE_PCC( var Base: TMatR; av: complex ): integer;
	end;

const
	RMatR: TStreamRec = (
	 ObjType: 4006;
	 VmtLink: Ofs(TypeOf(TMatR)^);
	 Load:    @TMatR.Load;
	 Store:   @TMatR.Store
	);


procedure RegisterMatReal;

implementation

procedure RegisterMatReal;
begin
	RegisterType(RVectR);
	RegisterType(RMatR);
end;



constructor TMatR.Init;
begin
 TMat.init(filas,columnas,sizeof(NReal));
end;

constructor TMatR.Load( var S: TStream );
begin
	TMat.Load(S);
end;

function TMatR.e;
begin
 e:=NReal(pte(k,j)^)
end;

procedure TMatR.pon_e(k,j: integer; x:NReal);
var
	p:pointer;
begin
	p:= pte(k,j);
	NReal(p^):=x;
end;

procedure TMatR.acum_e(k,j: integer; x:NReal);
var
	p:pointer;
begin
	p:= pte(k,j);
	NReal(p^):=NReal(p^)+x;
end;




constructor TVectR.Init(ne:word);
begin
 TVect.init(ne,sizeof(NReal));
end;

constructor TVectR.Load( var S: TStream );
begin
	TVect.Load(S);
end;

procedure TVectR.Print;
var
	k:integer;
begin
	writeln(' TVectR.print.inicio');
	TVect.Print;
	for k:= 1 to n do
		writeln(' N: ',k:6,' : ',e(k):12:4);
	writeln(' TVectR.print.fin');
end;


constructor TVectR.Ventana( ne: word; var x );
begin
	TVect.Ventana( ne, SizeOf(NReal), x);
end;

function TVectR.e(k:word):NReal;
begin
	e:=NReal(pte(k)^);
end;

procedure TVectR.pon_e(k:word; x:NReal);
var
	p:pointer;
begin
	p:= pte(k);
	NReal(p^):=x;
end;
procedure TVectR.acum_e(k:word; x:NReal);
var
	p:pointer;
begin
	p:= pte(k);
	NReal(p^):=x+NReal(p^);
end;


procedure TMatR.Ceros;
var
	k, j: integer;
begin
	for k:= 1 to nf do
		for j:= 1 to nc do pon_e(k,j, 0);
end;

procedure TMatR.MinMax(
			var kMin, jMin:word;
			var kMax, jMax:word;
			var Min,  Max:NReal);
var
	k, j: integer;
	m:NReal;
begin
	Min:=NReal(pte(1,1)^);
	Max:=Min;
	kmin:=1; jmin:=1; kmax:=1; jmax:=1;
	for k:= 1 to nf do
	begin
		for j:= 1 to nc do
		begin
			m:=NReal(pte(k,j)^);
			if m<Min then
			begin
				kmin:=k;
				jmin:=j;
				min:=m;
			end
			else
			if m>Max then
			begin
				kmax:=k;
				jmax:=j;
				max:=m;
			end;
		end;
	end;
end;






procedure TMatR.CerosFila( kfil: integer);
var
	k: integer;
begin
	for k:= 1 to nc do pon_e(kfil, k,0);
end;


	
procedure TVectR.Ceros;
var
	k:integer;
	p:pointer;
begin
	p:=pv;
	for k:= 1 to n do
	begin
		NReal(p^):=0;
		inc(p)
	end
end;

function TVectR.SumaDeElementos: NReal;
var
	k:integer;
	p:pointer;
	acum: NReal;
begin
	acum:=0;
	p:=pv;
	for k:= 1 to n do
	begin
		acum:= acum+ NReal(p^);
		inc(p)
	end;

end;



procedure TVectR.MinMax( var kMin, kMax: word; var Min, Max:NReal);
var
	k:integer;
	p:pointer;
begin
	p:=pv;
	Min:= NReal(p^); kMin:=1;
	Max:= NReal(p^); kMax:=1;
	for k:= 1 to n do
	begin
		if NReal(p^)<min then
		begin
			Min := NReal(p^);
			kMin:=k
		end
		else
			if NReal(p^) > max then
			begin
				Max := NReal(p^);
				kMax:= k
			end;
		inc(p)
	end
end;


(*

function TVectR.EstimFrec(
				var frecuencia: NReal; 	{ Estimacion de la frecuencia }
				nivel,						{ Nivel de compoaracion }
				histeresis: NReal 		{ Histeresis del cruce }
				): integer; 				{ Cantidad de cruces }





var
	Gatillado: integer;
	Periodos: LongInt;
	SumaDePeriodos: NReal;
	n1,n2: NReal;

function dknextcruce(var k, gatillo: integer): integer;
var
	k1, g1: integer;
begin
		k1:=k;
		g1:=gatillo;
		{ Busqueda del cambio de signo de gatillo }
		while (Gatillo =g1)and(k<=N) do
			if e(k) > n1 then Gatillo:=1
			else if e(k)<n2 then Gatillo:= -1
			else inc(k);
		if Gatillo=g1 then dknextcruce:=-1
		else
		begin
			g1:=gatillo;
			{ Busqueda del cambio de signo de gatillo }
			while (Gatillo =g1)and(k<=N) do
				if e(k) > n1 then Gatillo:=1
				else if e(k)<n2 then Gatillo:= -1
				else inc(k);
			if Gatillo=g1 then dknextcruce:=-1
			else dknextcruce:= k-k1;
		end;
end;

begin
	n1:=Nivel-Histeresis/2;
	n2:= n1+Histeresis;

	SumaDePeriodos:= 0;
	Periodos:= 0;
	k:=1;

	while (Gatillo = 0)and(k<=N) do
		if e(k) > n1 then Gatillo:=1
		else if e(k)<n2 then Gatillo:= -1
		else inc(k);
	if Gatillo = 0 then EstimFrec:=-1{ No podemos estimar con la histeresis }
	else
	begin
		Buscando:= true;
		while Buscando do
		begin
			dk:= dknextcruce( k, gatillo);
			if dk<0 then Buscando:=false
			else
			begin
				Periodos:= Periodos+1;
				SumaDePeriodos:= SumaDePeriodos+

*)


function TVectR.PEV(var y :TVectR):NReal;
var
 k:integer;
 p1,p2:pointer;
 temp:NReal;
begin
p1:=pv;p2:=y.pv;
temp:=0;
		 for k:=1 to n do
			  begin
				temp:=temp+NReal(p1^)*NReal(p2^);
			  inc(p1);y.inc(p2)
			  end;
PEV:=temp
end;  (* PEV *)


function TVectR.PEVRFLX(var y:TVectR): NReal;
var
 k:integer;
 p1,p2:pointer;
 temp:NReal;
begin
p1:=pv;p2:=y.pte(y.n);
temp:=0;
		 for k:=1 to n do
				begin
				temp:=temp+NReal(p1^)*NReal(p2^);
			  inc(p1);y.dec(p2)
			  end;
PEVRFLX:=temp
end;  (* PEVRFLX *)



procedure TVectR.Copy(var x:TVectR);
var
	k:integer;
begin
	move(x.pv^,pv^,n*SizeOf(NReal));
end;

procedure TVectR.sum(var y:TVectR);
var
	k:integer;
	p1,p2:pointer;
begin
	p1:=pv; p2:=y.pv;
	for k:=1 to n do
	begin
		NReal(p1^):=NReal(p1^)+NReal(p2^);
		inc(p1); y.inc(p2)
	end
end;

procedure TVectR.sumRPV(r:NReal;var x:TVectR);
var
	k:integer;
	p1,p2:pointer;
begin
	p1:=pv; p2:=x.pv;
	for k:=1 to n do
	begin
		NReal(p1^):=NReal(p1^)+r*NReal(p2^);
		inc(p1); x.inc(p2)
	end
end;


procedure TVectR.PorReal(r:NReal);
var
	k:integer;
	p1:pointer;
begin
	p1:=pv;
	for k:=1 to n do
	begin
		NReal(p1^):=NReal(p1^)*r;
		inc(p1)
	end;
end;

function TVectR.ne2:NReal; {norma euclideana al cuadrado }
var
	k:integer;
	p1:pointer;
	acum:NReal;
begin
	p1:=pv;
	acum:=0;
	for k:= 1 to n do
	begin
		acum:=acum+SQR(NReal(p1^));
		inc(p1)
	end;
	ne2:=acum
end;

function TVectR.normEuclid:NReal;
begin
	normEuclid:=sqrt(ne2)
end;

function TVectR.normMaxAbs:NReal;
var
	k:integer;
	p1:pointer;
	maxA:NReal;
begin
	p1:=pv;
	maxA:=0;
	for k:= 1 to n do
	begin
		if ABS(NReal(p1^))> maxA then maxA:=Abs(NReal(p1^));
		inc(p1)
	end;
	normMaxAbs:=maxA
end;

function TVectR.normSumAbs:NReal;
var
	k:integer;
	p1:pointer;
	acum:NReal;
begin
	p1:=pv;
	acum:=0;
	for k:= 1 to n do
	begin
		acum:=acum+ABS(NReal(p1^));
		inc(p1)
	end;
	normSumAbs:=acum
end;

procedure TMatR.WriteM;
var
   k,J:word;
begin
writeln;
writeln('---------------------------------------');
for k:=1 to nf do
	begin
	write('fila',k:3,'):');
	for j:=1 to nc do
		write(e(k,j):12:4);
	writeln;
	end;
end;


constructor TMatR.ReadM; (* a debe estar sin inicializar *)
var
   k,J:word;
begin
writeln;
writeln('---------------------------------------');
write('numero de filas=?');readln(k);
write('numero de columnas=?');readln(j);
init(k,j);
for k:=1 to nf do
	begin
	write('fila',k:3,'):?');
	for j:=1 to nc do
		read(NReal(pte(k,j)^));
	writeln;
	end;
end;



function TMatR.Escaler(var i:TMatR):NReal;
{$ifdef testdeter }
procedure muestre;
begin
	writeM;
	i.writeM;
	readln;writeln('===============');
end;
{$endif}

var
 k,p,j:integer;
 ptfp,ptfe:pointer;
 det,m,mc1:NReal;

begin
p:=1;det:=1;
{esca1}
while p<n do
	begin
	{$ifdef testdeter }
	muestre;
	{$endif}

	m:=abs(e(p,p));j:=p;

	for k:=p+1 to n do
	begin
		ptfp:=pte(k,p);
		if abs(NReal(ptfp^))>m then
		begin
			m:=abs(NReal(ptfp^));j:=k
		end;
	end;

	if p<>j then
		begin
		IntercambieFilas(p,j);
		i.IntercambieFilas(p,j);
		det:=-det
		end;

	if m=0 then
		begin
		det:=0;
		p:=nf
		end
	else{eliminacion}
		begin
		mc1:=e(p,p);
		det:=det*mc1;
		for k:=p+1 to nf do
			begin
			ptfp:=pte(p,p);
			ptfe:=pte(k,p);
			m:=-NReal(ptfe^)/mc1;
			for j:=p+1 to nc do
				begin
				incol(ptfp);incol(ptfe);
				NReal(ptfe^):=NReal(ptfe^)+m*NReal(ptfp^);
				end;
			ptfp:=i.pte(p,1);
			ptfe:=i.pte(k,1);
			for j:=1 to i.nc do
				begin
				NReal(ptfe^):=NReal(ptfe^)+m*NReal(ptfp^);

				i.incol(ptfp);i.incol(ptfe);
				end;
			end
		end;
	p:=p+1;
	end;(* while *)
det:=det*e(n,n);

if det<>0 then
begin{esca2}

	for k:=1 to nf do
	begin
		{$ifdef testdeter }
		muestre;
		{$endif}
		mc1:=1/e(k,k);
		ptfp:=i.pte(k,1);
		for j:=1 to i.nc do
		begin
			NReal(ptfp^):=NReal(ptfp^)*mc1;
			i.incol(ptfp)
		end;
		ptfp:=pte(k,n);
		for j:=nc downto k+1 do
		begin
			NReal(ptfp^):=NReal(ptfp^)*mc1;
			decol(ptfp);
		end;
	end;

	for p:=nf downto 2 do
	for k:=p-1 downto 1 do
	begin
		mc1:=e(k,p);
		ptfe:=i.pte(k,1);
		ptfp:=i.pte(p,1);
		for j:=1 to i.nc do
		begin
			NReal(ptfe^):=NReal(ptfe^)-NReal(ptfp^)*mc1;
			i.incol(ptfp);i.incol(ptfe);
		end;
	end;

end;

Escaler:=det;
{$ifdef testdeter }
muestre;
{$endif}
end {deter};


procedure TMatR.CopyColVect(var Y: TVectR; J: Integer);
var
	k:integer;
	p1,p2:pointer;
begin
p1:=y.pv;
for k:=1 to nf do
	begin
	NReal(p1^):=NReal(pte(k,j)^);
	y.inc(p1)
	end;
end;  (* CopyColVect *)



function TMatR.Traza:NReal;
var
   k:integer;
	temp:NReal;
	p1:pointer;
begin
temp:=NReal(pte(1,1)^);
for k:=2 to nc do
	begin
	temp := temp + NReal(pte(k,k)^);
	end;
Traza:=temp
end; (* Traza *)

procedure TMatR.PolinomioCaracteristico(var P:TPoliR);

var
	 pr:NReal;
   k,j:integer;
   pj:pointer;
	 m:TMatR;
begin
	  m.init(nf,nc);
	  p.gr:=nc;
	  m.igual(Self);
	  pr :=m.Traza;
     p.c[p.gr]:=1;
     p.c[p.gr-1] :=-pr;
     for k:=2 to p.gr do
         begin
         for j:= 1 to p.gr do
             begin
             pj:=m.pte(j,j);
						 NReal(pj^):=NReal(pj^)-pr;
             end;
			m.Mult(m,self);
			pr:=m.Traza/k;
         p.c[p.gr-k]:=-pr;
         end;
     p.c[p.gr]:=1;
     m.done;
end; (* PolinomioCaracteristico *)


procedure TMatR.Mult(a,b:TMatR);
var
   k,j:integer;
	 v:TVectR;
	 mtemp:TMatR;
   pa,pb,pc:pointer;

begin

	v.init(b.nf);
	mtemp.init(a.nf,a.nc);
	mtemp.igual(a);

	for j:=1 to b.nc do
  begin
  	pa:=mtemp.pv;
		b.CopyColVect(v,j);
    for k:=1 to A.nf do
    begin
			NReal(pte(k,j)^):= v.PEV(TVectR(pa^));
		mtemp.inc(pa);
    end;
  end;
	mtemp.done;
	v.done;
end;  (* MultTMatR *)



procedure TMatR.Transformar( var y, x: TVectR );
var
   k:integer;
   pa:pointer;
begin
 	pa:= pv;
   for k:=1 to nf do
	begin
		y.pon_e(k, x.PEV(TVectR(pa^)));
		inc(pa);
   end;
end;  (* Transformar *)




function TMatR.NormMaxAbs: NReal;
var
	k: integer;
	m, ms: NReal;
	p: PVectR;
begin
	p:= TVect.pte(1);
	m:= p^.NormMaxAbs;
	for k:= 2 to nf do
	begin
		p:= TVect.pte(k);
		ms:= p^.NormMaxAbs;
		if ms > m then m:= ms;
	end;
	NormMaxAbs:= m;
end;


function TMatR.deter:NReal;
var
temp1,temp2:TMatR;
begin
temp1.init(nf,nc);
temp1.igual(Self);
temp2.init(nf,0);
deter:=temp1.escaler(temp2);
temp2.done;
temp1.done;
end;

procedure TMatR.inv;
var
temp:TMatR;
k,j:integer;
ptemp:pointer;
aux:NReal;
begin
	temp.init(nf,nc);
	for k:=1 to nf do
	begin
		ptemp:=temp.pte(k,1);
		for j:=1 to nc do
		begin
			if k=j then NReal(ptemp^):=1
			else NReal(ptemp^):=0;
			temp.incol(ptemp);
		end;
	end;
	aux:=Self.escaler(temp);
	Self.igual(temp);
	temp.done;
end;




{
OJO esta en desarrollo 
retorna la dimensin del subespacio NULO.
La matriz deve ser cuadrada (hasta que se revise el algoritmo }
function TMatR.CalcBSEN( var Base: TMatR ): integer;

type
	TAGE = array[1..1000] of integer;
	PAGE = ^TAGE;

var
	k,p,j:integer;
	itmp: integer;
 	ptfp,ptfe:pointer;
	det,m,mc1:NReal;
	redundancia: integer;
	pidx: PAGE;
	buscando: boolean;
	xCeroPivote: NReal;


begin
	p:=1;
	det:=1;
	redundancia:=0;

	{ Relajacin del cero de la mquina para tener encuenta acumulacion
	de errores. /La eleccion de este valor esta realmente complicada. }
	xCeroPivote:= 1e-7;

   { inicializacion del direccionador de columnas }
	GetMem(pidx, SizeOf( integer )*nc);
	for k:= 1 to nc do pidx^[k]:=k;

	{esca1}
	while p<=n-redundancia do
	begin
      repeat 
			{ busqueda del mejor pivote }
			m:=abs(e(p,pidx^[p]));j:=p;
			for k:=p+1 to n do
			begin
				ptfp:=pte(k,pidx^[p]);
				if abs(NReal(ptfp^))>m then
				begin
					m:=abs(NReal(ptfp^));j:=k
				end;
			end;

			{ si es necesario intercambiar las filas }
			if p<>j then
			begin
				IntercambieFilas(p,j);
				det:=-det
			end;

{			writeln( abs(m) ); readlN; }
			if Casi0(m, xCeroPivote) then    {  pivote nulo }
			begin
				det:=0;
				itmp:=pidx^[p];
				pidx^[p]:= pidx^[nc-redundancia];
				pidx^[nc-redundancia]:=itmp;
				redundancia:= redundancia+1;
            buscando:= true;
			end
			else buscando:= false;
		until not buscando or( p>(n-redundancia));

		if buscando then
		else{eliminacion}
		begin
			mc1:=e(p,pidx^[p]);
			det:=det*mc1;
			for k:=p+1 to nf do
			begin
				ptfe:=pte(k,pidx^[p]);
				m:=-NReal(ptfe^)/mc1;
				for j:=p+1 to nc do
				begin
					ptfp:=pte(p,pidx^[j]);
					ptfe:=pte(k,pidx^[j]);
					NReal(ptfe^):=NReal(ptfe^)+m*NReal(ptfp^);
				end;
			end
		end;
		p:=p+1; { incremento el pivote }
	end;(* while *)

	det:=det*e(n,n);

   { Resultado de la funcin }
	CalcBsen:=redundancia;

   Base.init(redundancia, nf);
	if redundancia= 0 then  exit; { nada que hacer }

	if redundancia < n then
	begin{esca2}

		for k:=1 to n-redundancia do
		begin
			mc1:=1/e(k,pidx^[k]);
			for j:=nc downto k+1 do
			begin
				ptfp:=pte(k,pidx^[j]);
				NReal(ptfp^):=NReal(ptfp^)*mc1;
			end;
		end;


		{ Escalerizacin  hacia arriba en el bloque no redundante
		modificando el bloque redundante }
		for p:=nf-redundancia downto 2 do
		for k:=p-1 downto 1 do
		begin
			mc1:=e(k,pidx^[p]);
			for j:=1 to redundancia do
			begin
				ptfe:=pte(k,pidx^[nf-redundancia+j]);
				ptfp:=pte(p,pidx^[nf-redundancia+j]);
				NReal(ptfe^):=NReal(ptfe^)-NReal(ptfp^)*mc1;
			end;
		end;


		{
		Un vector del subespacio nulo V lo partimos en dos V1 el conjunto
		de variables NO redundantes y V2 el conjunto de las redundantes
		debiendose cumplir:

		I V1 + B V2 = 0 => V1 = -BV2 .

		donde B es la matriz d (n-redundancia)x(redundancia) que queda en
		la esquina superior derecha de la matriz que estamos procesando.

		Observar que considerando la primer componente de V2 igual a 1 y
		todas las demas 0 y despejando el vector V1 correspondiente
		tenemos un vector del subespacio nulo. As con el mismo procedimiento
		sobre cada una de las (redundancia) componentes de V2 obtenemos
		una base del espacio nulo que es lo que estamos buscando. Por
		construccin los vectores son Linealmente Independientes (LI).
		}


      {Copia el resultado}
		for k:= 1 to redundancia do
		begin
			for j:= 1 to nf-redundancia do
				base.pon_e(k,j , -e(j, pidx^[nc-redundancia+k]));
			for j:= nf-redundancia+1 to nf do
				base.pon_e(k,j,0);
			base.pon_e(k, nc-k+1, 1);
		end;

	end;


	{ inicializacion del direccionador de columnas }
	FreeMem(pidx, SizeOf( integer )*nc);

end {deter};





{ Calcula una base del subespacio invariante asociado a un autovalor
real dado }
function TMatR.CalcBSE_R( var Base: TMatR; av: NReal): integer;
var
	k: integer;
begin
	{ Hacer coincidir el subespacio asociado a la raiz con el
	subespacio nulo }
	for k:= 1 to n do pon_e(k,k, e(k,k)-av);
	CalcBSE_R:= CalcBSEN( Base );
end;




{ Calcula una base del subespacio invariate asociado a un par de
autovalores complejos conjugados }
function TMatR.CalcBSE_PCC( var Base: TMatR; av: complex ): integer;
var
	k: integer;
	tm: TMatR;
   ed: NReal;
begin
	tm.init( nf, nc );
	tm.igual( Self);
   ed:= -2*av.r;
	for k:= 1 to n do tm.pon_e(k,k, tm.e(k,k)+ed);
	Mult(tm, Self);
	tm.done;
   ed:= av.r*av.r+av.i*av.i;
	for k:= 1 to n do tm.pon_e(k,k, tm.e(k,k)+ed);
	CalcBSE_PCC:= CalcBSEN( Base );
end;




procedure TVectR.HacerUnitario;
var
	m: NReal;
begin
	m:= NormEuclid;
	if Not EsCero(m/n) then PorReal(1/m);
end;


function TMatR.OrtoNormal: integer;

var
	k, j: integer;
	pf, pfj: PVectR;
	m: NReal;
   redundancia: integer;
begin
	redundancia:=0;
	for k:= 1 to nf do
   begin
		pf:=TVect.pte(k);
		m:= pf^.NormEuclid;
		if Not EsCero(m/nc) then
		begin
			pf^.PorReal(1/m);
   		{ ortogonalizacion del resto }
			for j:= k+1 to nf do
			begin
				pfj:= TVect.pte(j);
				m:= pf^.PEV(pfj^);
				pfj^.SumRPV(-m, pf^);
			end;
		end
		else redundancia:= redundancia+1;
	end;
	OrtoNormal:= nf-redundancia;
end;
   	

	




begin
(*
writeln('Unidad MatReal INSTALADA / RCH-90');
*)
end.